
#include <QtCore/QDateTime>
#include <QtCore/QTextStream>
#include <QtCore/QStringList>
#include <QtDBus/QDBusMetaType>

#include "service.h"

SystemInfo::SystemInfo(QObject *parent)
	: QDBusAbstractAdaptor(parent),
	  m_syslog(200),
	  m_dbusMonitor(50),
	  m_syslogFile("/var/log/syslog")
{
	if (m_syslogFile.open(QIODevice::ReadOnly)) {
		m_syslog.setDevice(&m_syslogFile);
	}

	m_dbusMonitorProcess.start("/usr/bin/dbus-monitor");
	m_dbusMonitor.setDevice(&m_dbusMonitorProcess);

	qDBusRegisterMetaType<QMap<QString, QVariant> >();
	startTimer(1000);
}

uint SystemInfo::Date()
{
	return QDateTime::currentDateTime().toTime_t();
}

void SystemInfo::Noop()
{
}

QString SystemInfo::Ping(QString text)
{
	return text;
}

int SystemInfo::CpuLoad()
{
	return m_cpuLoad;
}

void SystemInfo::timerEvent(QTimerEvent *)
{
	int cpuCount = -1;
	QFile file("/proc/stat");
	if (!file.open(QIODevice::ReadOnly))
		return;
	QTextStream stream(&file);
	do {
		QString line = stream.readLine();
		const QStringList list = line.trimmed().split(" ",  QString::SkipEmptyParts);
		if (list[0].left(3) == "cpu")
			++cpuCount;
		if ((list.count() < 6) || list[0] != "cpu")
			continue;

		int value = 0;
		for (int i = 0; i < list.length()-1; ++i) {
			if (i == 3) // idle
				continue;
			value += list[i+1].toInt();
		}
		m_cpuLoad = value - m_oldStat;
		m_oldStat = value;
	} while(!stream.atEnd());
	m_cpuLoad /= cpuCount;

	m_syslog.update();
	m_dbusMonitor.update();
}

Q_DECLARE_METATYPE(QStringList)

QMap<QString, QVariant> SystemInfo::Syslog(int last)
{
	QMap<QString, QVariant> ret;

	m_syslog.update();

	ret["last"] = QVariant(m_syslog.last());
	ret["data"] = QVariant(m_syslog.get(last));
	return ret;
}

QMap<QString, QVariant> SystemInfo::DBusMonitor(int last)
{
	QMap<QString, QVariant> ret;

	m_dbusMonitor.update();

	ret["last"] = QVariant(m_dbusMonitor.last());
	ret["data"] = QVariant(m_dbusMonitor.get(last));
	return ret;
}

